@@ -548,8 +548,11 @@ class Uploader(object):
                 timeout = self.timeout
             )
         except Exception as e:
+            if self.promise is None:
+                raise
             self.promise.reject(errors.Error('upload', self.url.path, 500, str(e)))
             return
+        response.encoding = 'utf-8'
 
         status = response.status_code
         result = http.decode_response_payload(response.text)
@@ -558,6 +561,8 @@ class Uploader(object):
             self.promise.reject(errors.Error('upload', self.url.path, status, result))
             return
 
+        if self.promise is None:
+            return result
         self.promise.resolve(result)
 
 class FileProgressUpload(object):
